/*
    (C) 1995-97 AROS - The Amiga Research OS
    $Id: assignadd.c,v 1.1.1.1 2005/03/15 15:54:40 laire Exp $

    Desc: Add a directory to an assign.
    Lang: English
*/
#include <exec/memory.h>
#include <proto/exec.h>
#include <dos/dos.h>
#include <dos/dosextens.h>
#include "dos_intern.h"

/*****************************************************************************

    NAME */
#include <proto/dos.h>

	AROS_LH2(LONG, AssignAdd,

/*  SYNOPSIS */
	AROS_LHA(CONST_STRPTR, name, D1),
	AROS_LHA(BPTR  , lock, D2),

/*  LOCATION */
	struct DosLibrary *, DOSBase, 105, Dos)

/*  FUNCTION
	Create a multi-directory assign, or adds to it if it already was one.
	Do not use or free the lock after calling this function - it becomes
	the assign and will be freed by the system when the assign is removed.

    INPUTS
	name - NULL terminated name of the assign.
	lock - Lock on the assigned directory.

    RESULT
	!= 0 success, 0 on failure. IoErr() gives additional information
	in that case. The lock is not freed on failure.

    NOTES
	This will only work with an assign created with AssignLock() or
	a resolved AssignLate() assign.

    EXAMPLE

    BUGS

    SEE ALSO
	Lock(), AssignLock(), AssignPath(), AssignLate(), DupLock(),
	RemAssignList()

    INTERNALS

    HISTORY
	27-11-96    digulla automatically created from
			    dos_lib.fd and clib/dos_protos.h

*****************************************************************************/
{
    AROS_LIBFUNC_INIT
    AROS_LIBBASE_EXT_DECL(struct DosLibrary *,DOSBase)
    struct DosList     *dl;
    struct AssignList **al, *newal;

    DEBUG_ASSIGN(dprintf("AssignAdd: name = \"%s\", lock = $%lx\n", name, lock));

    FAILFORTASKNP(return DOSFALSE;)

    if (lock == NULL)
    {
	DEBUG_ASSIGN(dprintf("AssignAdd: fail (null)\n"));
	return DOSFALSE;
    }

    dl = LockDosList(LDF_ASSIGNS | LDF_WRITE);
    dl = FindDosEntry(dl, name, LDF_ASSIGNS);

    if ((dl) && (dl->dol_Type == DLT_LATE))
    {
	BPTR latelock = 0;

	/*
	 * For late assigns we need to resolve them first.
	 */
	DEBUG_ASSIGN(dprintf("AssignAdd: Resolve LATE Assign\n"));

	latelock = Lock(dl->dol_misc.dol_assign.dol_AssignName, ACCESS_READ);
	DEBUG_ASSIGN(dprintf("AssignAdd: Lock(\"%s\") = %lx\n", dl->dol_misc.dol_assign.dol_AssignName, latelock));
	if (latelock)
	{
	    struct FileLock *fl;
	    DEBUG_ASSIGN(dprintf("GetDeviceProc: Late Binding resolved\n"));
	    dosFreeVec(dl->dol_misc.dol_assign.dol_AssignName);
	    dl->dol_Type = DLT_DIRECTORY;
	    dl->dol_Lock = latelock;
	    fl = BADDR(latelock);
	    dl->dol_Task = fl->fl_Task;
	}
	else
	{
	    DEBUG_ASSIGN(dprintf("AssignAdd: couldn't resolve Late Binding\n"));
	}
    }

    if ((dl == NULL) || (dl->dol_Type != DLT_DIRECTORY))
    {
	DEBUG_ASSIGN(dprintf("AssignAdd: fail (wrong type)\n"));

	UnLockDosList(LDF_ASSIGNS | LDF_WRITE);
	SetIoErr(ERROR_OBJECT_WRONG_TYPE);

	return DOSFALSE;
    }
    
    newal = dosAllocVec(sizeof(struct AssignList), MEMF_PUBLIC | MEMF_CLEAR);

    if (newal == NULL)
    {
	DEBUG_ASSIGN(dprintf("AssignAdd: fail (no mem)\n"));

	UnLockDosList(LDF_ASSIGNS | LDF_WRITE);
	SetIoErr(ERROR_NO_FREE_STORE);

	return DOSFALSE;
    }
    
    newal->al_Lock = lock;

    for(al = &dl->dol_misc.dol_assign.dol_List; *al; al = &((*al)->al_Next));

    *al = newal;
    UnLockDosList(LDF_ASSIGNS | LDF_WRITE);

    return DOSTRUE;

    AROS_LIBFUNC_EXIT
} /* AssignAdd */
